
//Sketch for QRPkits.com Si5351 VFO 
//Generic with QRP default frequencies
//Written by KD1JV 
//si5351 routine by Jerry Gaffke, KE7ER


#include <EEPROM.h>
#include "Wire.h"


// si5451 definations 
#define BB0(x) ((uint8_t)x)             // Bust int32 into Bytes
#define BB1(x) ((uint8_t)(x>>8))
#define BB2(x) ((uint8_t)(x>>16))

#define SI5351BX_ADDR 0x60              // I2C address of Si5351   (typical)
#define SI5351BX_XTALPF 3             // 1:6pf  2:8pf  3:10pf

#define SI5351BX_XTAL 25000000          // Crystal freq in Hz
#define SI5351BX_MSA  35                // VCOA is at 25mhz*35 = 875mhz

// User program may have reason to poke new values into these 3 RAM variables
uint32_t si5351bx_vcoa = (SI5351BX_XTAL*SI5351BX_MSA);  // 25mhzXtal calibrate
uint8_t  si5351bx_rdiv = 0;             // 0-7, CLK pin sees fout/(2**rdiv) 
uint8_t  si5351bx_drive[3] = {0,0,0};   // 0=2ma 1=4ma 2=6ma 3=8ma for CLK 0,1,2

uint8_t  si5351bx_clken = 0xFF;         // Private, all CLK output drivers off


//Si5351 si5351;
#define  SLED5  9
#define  SLED4  10
#define  SLED3  11
#define  SLED2  12
#define  SLED1  13


#define   LED_N_0  0xde
#define   LED_N_1  0x12
#define   LED_N_2  0xd5
#define   LED_N_3  0x57
#define   LED_N_4  0x1b
#define   LED_N_5  0x4f
#define   LED_N_6  0xcb
#define   LED_N_7  0x16
#define   LED_N_8  0xdf
#define   LED_N_9  0x1f
#define   LED_r    0x81
#define   LED_neg   0x01
#define   LED_C     0xcc
#define   LED_n     0x83
#define   LED_E     0xcd
#define   LED_F     0x8d
#define   LED_t     0xc9
#define   LED_o     0xc3 
#define   LED_d     0xd3
#define   LED_A     0x9f
#define   LED_L     0xc8 
#define   LED_P     0x9d
#define   LED_BLANK 0x00
#define   LED_question 0xbd
#define   LED_point 0x20


//flag definitions 
#define     E_sw    2
#define     U_sw    0
#define     D_sw    1
#define     ENC_A   A0
#define     ENC_B   A1
#define     RIT_ON 0x01
#define     RIT_FC 0xfe


// register names 
byte        SRtemp;
byte        Etemp; 
byte        BANDpointer; 
byte        outofband =0; 
byte        ritflag = 0;
byte        EncoderFlag = 0;
byte        c = 0;
byte        d = 0;
byte        cLast = 1; 
byte        shift_display =0;  
unsigned long        duration;

int                Eadr;
volatile byte      sw_inputs ; 


volatile byte      digit1 = 0xff;
volatile byte      digit2 = 0xff;
volatile byte      digit3 = 0xff;
volatile byte      digit4 = 0xff;
volatile byte      digit5 = 0xff;
volatile byte      digitX = 0;

volatile byte      d1temp = 0;

volatile byte    digit_counter = 1;
    //frequency tuning 
int         stepSize;   // tuning rate pointer 
int         tempSS; 
long int    stepK;      // temp storage of freq tuning step
long int    tempSK; 

long int    Fstep10  = 10 ;                // 10 Hz step
long int    Fstep100 = 100;
long int    Fstep1K =  1000;
long int    Fstep5K =  5000;
long int    Fstep10K = 10000;
long int    Fstep100K = 100000;
                                          
//encoder 


unsigned  long  temp1;
unsigned  long frequency;
unsigned  long freq_result;  
unsigned  long OPfreq;
volatile unsigned long   tcount;
unsigned long   time0;
unsigned long   temp ; 
unsigned long   cal_value;
unsigned long  IF_offset =0; 
unsigned long  RX_freq =0;
unsigned long  RITtemp;
 //********************************************** 

// registers for limit testing

unsigned long  low_band_limit;  //low limit, band tuning
unsigned long  high_band_limit; //high limit, band tuning 
unsigned long  low_absolute_limit = 500000;
unsigned long  high_absolute_limit = 30000000;

//temp band freq registers

unsigned long band_temp_160 = 0; 
unsigned long band_temp_80 = 0;
unsigned long band_temp_60 = 0;
unsigned long band_temp_40 = 0;
unsigned long band_temp_30 = 0;
unsigned long band_temp_20 = 0;
unsigned long band_temp_17 = 0;
unsigned long band_temp_15 = 0;
unsigned long band_temp_12 = 0;
unsigned long band_temp_10 = 0;
unsigned long band_temp_06 = 0;

void setup() {
 //switch inputs

DDRB = 0xff;
DDRD = 0Xff; 
pinMode(A3,INPUT_PULLUP);
pinMode(A0, INPUT_PULLUP);
pinMode(A1, INPUT_PULLUP);


     
      noInterrupts();
    TCCR1A = 0;
    TCCR1B = 0;
    ASSR = 0;
    TCNT1 = 0;

    OCR1A = 62;    // 1ms interval
    TCCR1B = 0x0c;  // WGM22=0, CS=0b100 (16MHz / 128 = 2MHz)
    TIFR1 = 0x07;   // clear interrupt flags
    TIMSK1 |= (1 << OCIE1A);  // enable Output Compare Match A Interrupt
      interrupts();   

      
      si5351bx_init(); 
      cal_data();    //load calibration data 
      IF_DATA(); 
      stepK = Fstep1K ; //default tuning rate
      stepSize = 3; 
      delay(1000); 
      int_band();
      delay(1000); //let things settle down a bit
      displayfreq();
      PLLwrite();
}


void loop() {
  // test for switch closed
        if (digitalRead(A3) == LOW) {transmit();}    
        if (bitRead(sw_inputs, E_sw) == LOW) {timedswitch(); 
        debounceE();}         
        if (bitRead(sw_inputs, U_sw) == LOW) {Tune_UP();} //test tune up flag
        if (bitRead(sw_inputs, D_sw) == LOW) {Tune_DWN();} //test tune down flag   
        if (EncoderFlag == 1) {Tune_UP();} //test tune up flag
        if (EncoderFlag == 2) {Tune_DWN();} //test tunr down flag 

}   


//**************************************************************
//end of switch polling loop
//**************************************************************

void debounceE(){
    while (bitRead(sw_inputs, E_sw) == LOW) {loop;} 
    delay(20);
    }

void debounceU(){
    while (bitRead(sw_inputs, U_sw) == LOW) {loop;} 
}

void debounceD(){
    while (bitRead(sw_inputs, D_sw) == LOW) {loop;} 
}

void timedswitch(){ 

    tcount = 0; 
     
  do { 

        if (tcount == 1000) { //rit mode
            digit5 = LED_r;
            digit4 = LED_BLANK;
            digit3 = LED_BLANK;
            digit2 = LED_BLANK;
            digit1 = LED_BLANK;
        } 
      
       if (tcount == 2000) {
        // dispaly shift
            digit5 = LED_d;
            digit4 = LED_N_5;
            digit3 = LED_BLANK;
            digit2 = LED_BLANK;
            digit1 = LED_BLANK;
        }  
        
      if  (tcount == 3000) {
        //change bands
          digit5 = LED_N_6; 
          digit4 = LED_n; 
          digit3 = 0x00; 
          digit2 = 0x00; 
          digit1 = 0x00;
       }

      if  (tcount == 10000) {
          //store if freq
          digit5 = LED_N_5; 
          digit4 = LED_neg; 
          digit3 = LED_N_1; 
          digit2 = LED_F; 
          digit1 = 0x00;
      }
        
      if (tcount== 12000) {
        //calibration 
          digit5 = LED_C; 
          digit4 = LED_A; 
          digit3 = LED_L; 
          digit2= 0x00; 
          digit1 = 0x00;
        }
        
       if (tcount == 14000) { 
        //clear eerom 
          digit5 = LED_r;
          digit4 = LED_E;
          digit3 = LED_N_5;
          digit2 = LED_BLANK;
          digit1 = LED_BLANK;
        }
      
  } while (bitRead(sw_inputs,E_sw) !=1);
 
    if (tcount >= 14000) {
        reset_flash();
    }
    
    else if (tcount >= 12000) {
        calibration();
    }

    else if (tcount >=10000){
      store_if();
    }

    else if (tcount >= 3000) {
       changeBand();
    }

    else if (tcount >=2000) {
      toggleDS();
    }

    else if (tcount >= 1000)  {
        RIT();
    }   

    else if (tcount >= 10) {
        incStep();
    }
}

void  toggleDS(){

    if (shift_display == 1) {shift_display = 0; }
    else shift_display = 1;
    stepSize = 3; 
    stepK = Fstep1K;
    displayfreq();
    debounceE(); 
}

void  Tune_UP() {
      FREQ_incerment();
      delay(150);
}

void Tune_DWN() {    
      FREQ_decerment();
      delay(150);    
}


// adjust the operating frequency
void FREQ_incerment() {
      EncoderFlag = 0;
      if (OPfreq >= high_absolute_limit) {return;} 
      OPfreq  = OPfreq + stepK;  //add frequenc tuning step to frequency word
      outofband = 0;
      Current_freq_temp();
      if (OPfreq > high_band_limit) {outofband =1;} //band tuning limits
      if (ritflag==1){RITdisplay();}
      else {displayfreq();}
      PLLwrite();    
}

void FREQ_decerment() {
      EncoderFlag = 0; 
      if (OPfreq <= low_absolute_limit) {return;} 
      OPfreq  = OPfreq - stepK;
      outofband = 0;
      Current_freq_temp();
      if (OPfreq < low_band_limit) {outofband = 1;} 
      if (ritflag==1){RITdisplay();}
      else {displayfreq();}
      PLLwrite(); 
}

void Current_freq_temp() {
switch(BANDpointer){
case 1: band_temp_160 = OPfreq;
        break;
case 2:  band_temp_80 = OPfreq;
        break;      
case 3: band_temp_60 = OPfreq;
        break;
case 4: band_temp_40 = OPfreq;
        break;
case 5: band_temp_30 = OPfreq;
        break;        
case 6: band_temp_20 = OPfreq;
        break; 
case 7: band_temp_17 = OPfreq;
        break;
case 8: band_temp_15 = OPfreq;
        break;
case 9: band_temp_12 = OPfreq;
        break;
case 10: band_temp_10 = OPfreq;
        break;
case 11: band_temp_06 = OPfreq;
        break;        
}
}



void incStep() {
    if (ritflag ==1) {ritSteps();}
    else normal_shift();
}


void normal_shift(){     
    if (shift_display == 1) {allsteps();} 
    if  (shift_display == 0) {bigsteps();}
}


void ritSteps() {
   ++ stepSize ;
    if (stepSize == 4) {(stepSize = 1);}
    
switch(stepSize) {
     case 1:
          stepK = Fstep10;
          d1temp = digit1;
          digit1 = 0x00;    
          delay(100); 
          digit1 = d1temp;
          break;
           
    case 2:
          stepK = Fstep100;
          d1temp = digit2;
          digit2 = 0x00;
          delay(100); 
          digit2 = d1temp;
          break; 

    case 3:   
          stepK = Fstep1K;
          d1temp = digit3;
          digit3 = 0x00;  
          delay(100); 
          digit3 = d1temp;
          break; 
          }
          debounceE();
          
          }
          


void allsteps() {
  
    ++ stepSize ;
    if (stepSize == 6) {(stepSize = 1);} 
    
switch(stepSize) {
     case 1:
          stepK = Fstep10;
          d1temp = digit1;
          digit1 = 0x00;    
          delay(100); 
          digit1 = d1temp;
          break;
           
    case 2:
          stepK = Fstep100;
          d1temp = digit2;
          digit2 = 0x00;
          delay(100); 
          digit2 = d1temp;
          break; 

    case 3:   
          stepK = Fstep1K;
          d1temp = digit3;
          digit3 = 0x00;  
          delay(100); 
          digit3 = d1temp;
          break; 

     case 4: 
          stepK = Fstep5K;
          d1temp = digit3;
          digit3 = 0x00;  
          delay(100); 
          digit3 = d1temp;
          delay(100);
          digit3 = 0x00;
          delay(100);
          digit3 = d1temp;
          break;    

    case 5:                 
          stepK = Fstep10K;
          d1temp = digit4;
          digit4 = 0x00;  
          delay(100); 
          digit4 = d1temp;
          break;      
                }
        debounceE();              
}

void bigsteps() {
    ++ stepSize ;
    if (stepSize == 7) {(stepSize = 3);}
    
switch(stepSize) {
    case 1: break;
    case 2: break;
    case 3: 
          stepK = Fstep1K;
          d1temp = digit1;
          digit1 = 0x00;  
          delay(100); 
          digit1 = d1temp;
          break;   
    case 4: 
          stepK = Fstep5K;
          d1temp = digit1;
          digit1 = 0x00;  
          delay(100); 
          digit1 = d1temp;
          delay(100);
          digit1 = 0x00;
          delay(100);
          digit1 = d1temp;
          break;    

    case 5:
          stepK = Fstep10K;
          d1temp = digit2;
          digit2 = 0x00;  
          delay(100); 
          digit2 = d1temp;
          break;

    case 6:
          stepK = Fstep100K;
          d1temp = digit3;
          digit3 = 0x00;  
          delay(100); 
          digit3 = d1temp;
          break;
}
debounceE();
}

void RIT() {
    if (ritflag == 1) {
        RIText();
    }
    else {
        RITenable();
    }
}

void  RITenable(){
      ritflag = 1;
      tempSS = stepSize;
      tempSK = stepK; 
      stepSize = 1;
      stepK = 10; 
      RITtemp = OPfreq;
      PLLwrite();
      RITdisplay();
      debounceE();
}

void  RIText() {
      ritflag =0;
      stepK = tempSK;
      stepSize = tempSS; 
      OPfreq = RITtemp;
      PLLwrite();
      displayfreq();
      debounceE();
}

void RITdisplay() {
    unsigned long RITresult;
    if (RITtemp >= OPfreq) {
        RITresult = RITtemp - OPfreq;
        if (RITresult >= 10000){FREQ_incerment();}
        digit4 = LED_neg;
    }
    else {
        RITresult = OPfreq - RITtemp; 
       if (RITresult >= 10000){FREQ_decerment();}
        digit4 = LED_BLANK;   
    }

    frequency = RITresult;
    freq_result = frequency%10000;
    freq_result = freq_result/1000;
    hex2seg(); 
    digitX = digitX + LED_point;
    digit3 = digitX;
    freq_result = frequency%1000;
    freq_result = freq_result/100;
    hex2seg(); 
    digit2 = digitX;
    freq_result = frequency%100;
    freq_result = freq_result/10;
    hex2seg();
    digit1 = digitX;    
}

////////////////////////////////////////////////
//This breaks up the frequency value into decades and then converts the result 
//hex value into the LED 7 seg map. 
///////////////////////////////////////////////

void    displayfreq()
{

   if (shift_display == 1){FREQdisplay(); return;}
   else {OFB_F_display();}
}

   void OFB_F_display()

{
          frequency = OPfreq;    
          freq_result = frequency/10000000;
          hex2seg();
          if (freq_result == 0){digitX= 0x00;}
          digit5 = digitX;

          freq_result = frequency%10000000;
          freq_result = freq_result/1000000;
          hex2seg();
          digitX = digitX + 0x20;
          digit4 = digitX;
         
        freq_result = frequency%1000000;     //get the 100,000 kHz digit by first getting the remainder 
        freq_result = freq_result / 100000;  //divide the remainder by 100,000 to get the MSD
        hex2seg();                           //convert the result to the 7 segment code
        digit3 = digitX;  
        
        freq_result = frequency%100000;      //repeat the process for 10K, 1K and 100 Hz digits
        freq_result = freq_result/10000;
        hex2seg();
        digit2 = digitX;
        
        freq_result = frequency%10000;
        freq_result = freq_result/1000;
        hex2seg(); 
        digit1 = digitX;
}


void FREQdisplay(){
        frequency = OPfreq;              
        freq_result = frequency%1000000;     //get the 100,000 kHz digit by first getting the remainder 
        freq_result = freq_result / 100000;  //divide the remainder by 100,000 to get the MSD
        hex2seg();                           //convert the result to the 7 segment code
        if (freq_result == 0){digitX = 0x00;}
        digit5 = digitX;  
        
        freq_result = frequency%100000;      //repeat the process for 10K, 1K and 100 Hz digits
        freq_result = freq_result/10000;
        hex2seg();
        digit4 = digitX;
        
        freq_result = frequency%10000;
        freq_result = freq_result/1000;
        hex2seg(); 
        digitX = digitX+ 0x20;
        digit3 = digitX;
        
        freq_result = frequency%1000;
        freq_result = freq_result/100;
        hex2seg();
        digit2 = digitX;
        
        freq_result = frequency%100;
        freq_result = freq_result/10;
        hex2seg();
        digit1 = digitX;
}

void  hex2seg()
{
        if (freq_result == 0) {digitX = LED_N_0;} //this is the conversion table
        if (freq_result == 1) {digitX = LED_N_1;}
        if (freq_result == 2) {digitX = LED_N_2;}
        if (freq_result == 3) {digitX = LED_N_3;}
        if (freq_result == 4) {digitX = LED_N_4;}
        if (freq_result == 5) {digitX = LED_N_5;}
        if (freq_result == 6) {digitX = LED_N_6;}
        if (freq_result == 7) {digitX = LED_N_7;}
        if (freq_result == 8) {digitX = LED_N_8;}
        if (freq_result == 9) {digitX = LED_N_9;}
}

/*
 * 
 * timer outside of the normal Ardinu timers
 * does keyer timing and port D mulitplexing for display and 
 * switch inputs. 
 */

ISR(TIMER1_COMPA_vect){
    
    ++tcount;

    if (++digit_counter > 4) {
        digit_counter = 0;
    }

    switch(digit_counter) {
    
    case 0: 
        digitalWrite(SLED5, HIGH);
        readswitches(); 
        PORTD = digit1;
        digitalWrite(SLED1, LOW);
        break;

    case 1: 
        digitalWrite(SLED1, HIGH);
        readswitches(); 
        PORTD = digit2;
        digitalWrite(SLED2, LOW);
        break;

    case 2:  
        digitalWrite(SLED2, HIGH);
        readswitches();  
        PORTD = digit3;
        digitalWrite(SLED3, LOW);
        break;

    case 3: 
        digitalWrite(SLED3, HIGH);
        readswitches(); 
        PORTD = digit4;
        digitalWrite(SLED4, LOW);
        break;

    case 4: 
        digitalWrite(SLED4, HIGH);
        readswitches(); 
        PORTD = digit5;
        digitalWrite(SLED5, LOW);
        break;
    }
}

void readswitches() {                         //read the switches and encoder here
               
              DDRD = 0x00;
              PORTD= 0Xff; 
            digitalWrite(8, LOW); 
            sw_inputs = PIND; 
            digitalWrite(8, HIGH); 
            DDRD =0xff;
            c = digitalRead(ENC_A);  //read encoder clock bit
            if (c !=cLast){encoder();} //call if changed        
        }
                                 
     // encoder, test for direction only on 0 to 1 clock state change
     
void encoder()      {
        if (cLast ==0){
        d = digitalRead(ENC_B);    
          if ( d == LOW ) {EncoderFlag = 2;}          //if low
        
        else {EncoderFlag = 1;}          //if high
              
                       }
         cLast = c;            //store new state of clock  
                        }
/*
 * output the frequency data to the clock chip.
 */

void PLLwrite() {   
      RX_freq = OPfreq+IF_offset;
      si5351bx_setfreq(0,RX_freq); //update clock chip with VFO frequency.
}

void transmit() {
        if (outofband == 1){return;} 
        if (ritflag == 0) {si5351bx_setfreq(1, OPfreq);}
        else  si5351bx_setfreq(1, RITtemp); 
        tone(A2,600);
        while (digitalRead(A3) == LOW){loop;} 
        noTone(A2); 
        i2cWrite(3, 0xfe);        // Enable/disable clock 
}


void calibration(){

  displayfreq();
  debounceE();
  digit5 = LED_C;
 
while (bitRead (sw_inputs,E_sw)== HIGH)
{
  if   (bitRead(sw_inputs, U_sw) == LOW){ADJ_UP();}
  if   (bitRead(sw_inputs, D_sw) == LOW) {ADJ_DWN();} 
}

    temp = si5351bx_vcoa;  //store the cal value 
    cal_value = temp;  //store the cal value 
    EEPROM.write(7, temp);
    temp = cal_value >>8;
    EEPROM.write(6, temp); 
    temp = cal_value >>16;
    EEPROM.write(5, temp);
    temp  = cal_value >>24;
    EEPROM.write(4,temp); 

    displayfreq();  
    debounceE(); 
   
} //end of calibration routine



void ADJ_UP(){
      si5351bx_vcoa = si5351bx_vcoa -175;
      calwrite();
}

void ADJ_DWN() {
      si5351bx_vcoa = si5351bx_vcoa +175;
      calwrite();
}

void calwrite() {
      si5351bx_setfreq(1,OPfreq); 
      delay(150);
}

void cal_data(){
      temp1 = si5351bx_vcoa;
      temp =0; 
      temp = EEPROM.read(4);
      cal_value = temp;
      cal_value = cal_value << 8;
      temp = EEPROM.read(5);
      cal_value = cal_value + temp;
      cal_value = cal_value << 8;
      temp = EEPROM.read(6);
      cal_value = cal_value + temp;
      cal_value = cal_value << 8;
      temp = EEPROM.read(7);
      cal_value = cal_value + temp;
      si5351bx_vcoa = cal_value;
      if (cal_value == 0xffffffff) si5351bx_vcoa = temp1;
}

void store_if() {
    IF_offset = OPfreq;
    temp = OPfreq;  //store the cal value 
    EEPROM.write(11, temp);
    temp = OPfreq >>8;
    EEPROM.write(10, temp); 
    temp = OPfreq >>16;
    EEPROM.write(9, temp);
    temp  = OPfreq >>24;
    EEPROM.write(8,temp); 
    debounceE();
    delay(10);
    displayfreq();
    PLLwrite();
    debounceE(); 
}

void IF_DATA(){

      temp = EEPROM.read(8);
      IF_offset = temp;
      IF_offset = IF_offset << 8;
      temp = EEPROM.read(9);
      IF_offset = IF_offset + temp;
      IF_offset = IF_offset << 8;
      temp = EEPROM.read(10);
      IF_offset = IF_offset + temp;
      IF_offset = IF_offset << 8;
      temp = EEPROM.read(11);
      IF_offset = IF_offset + temp;
      if (IF_offset == 0xffffffff) IF_offset = 0;
}

void changeBand(){
      digit5 = LED_N_6;
      digit4 = LED_n; 
      digit3 = 0x00; 
      get_band();
    
    debounceE(); 
      
    do {
      if (bitRead(sw_inputs,U_sw) !=1) {nextband();}
      if (bitRead(sw_inputs,D_sw) !=1) {nextbandD();}
    }
    while (bitRead(sw_inputs,E_sw) !=0);

    EEPROM.write(0,BANDpointer); 
    
    debounceE(); 
    displayfreq();
    PLLwrite();  
}

void nextband () {
    ++BANDpointer ;
    if (BANDpointer == 12) {(BANDpointer = 1);}
    get_band();
    debounceU(); 
}

void nextbandD() {
   --BANDpointer ;
    if (BANDpointer == 0) {(BANDpointer = 11);}
    get_band();
    debounceD(); 
}

void int_band() {
  BANDpointer = EEPROM.read(0);
  if (BANDpointer == 0xff) {BANDpointer= 1; changeBand();}
  digit5 = LED_N_6;
  digit4 = LED_n; 
  digit3 = 0x00;
  get_band(); 
}


void get_band() {
switch(BANDpointer) {
case 1: 
      BAND16();
      break;
      
case 2:
      BAND80();
      break;

case 3: 
      BAND60() ;
      break ;
      
case 4: 
      BAND40() ;
      break ;
      
case 5: 
      BAND30() ;
      break ;
      
case 6: 
      BAND20() ;
      break ;
      
case 7: 
      BAND17() ;
      break ;

case 8:
      BAND15() ;
      break;

case 9:
      BAND12();
      break;

case 10:
      BAND10();
      break;

case 11:
      BAND06();      
}
}

void BAND16() {  
    digit2 = LED_N_1;
    digit1 = LED_N_6;
    low_band_limit = 1800000;
    high_band_limit = 2000000;
    if (band_temp_160 != 0) {OPfreq = band_temp_160;}
    else OPfreq = 1810000;    
}

void BAND80() {  
    digit2 = LED_N_8;
    digit1 = LED_N_0;
    low_band_limit = 3500000;
    high_band_limit = 4000000;
    if (band_temp_80 != 0) {OPfreq = band_temp_80;}
    else OPfreq = 3560000;    
}

void BAND60() {  
    digit2 = LED_N_6;
    digit1 = LED_N_0;
    low_band_limit = 5351500;
    high_band_limit = 5366500;
    if (band_temp_60 != 0) {OPfreq = band_temp_60;}
    else OPfreq = 5357000;    
}

void BAND40() { 
    digit2 = LED_N_4;  
    digit1 = LED_N_0; 
    low_band_limit = 7000000;
    high_band_limit = 7300000;
    if (band_temp_40 != 0) {OPfreq = band_temp_40;}
    else OPfreq = 7030000;
}

void BAND30() {  
    digit2 = LED_N_3;
    digit1 = LED_N_0;
    low_band_limit = 10100000;
    high_band_limit = 10150000;
    if (band_temp_30 != 0) {OPfreq = band_temp_30;}
    else OPfreq = 10106000; 
}

void BAND20() {  
    digit2 = LED_N_2;
    digit1 = LED_N_0;
    low_band_limit = 14000000;
    high_band_limit = 14500000;
    if (band_temp_20 != 0) {OPfreq = band_temp_20;}
    else OPfreq = 14060000;
}

void BAND17() {  
    digit2 = LED_N_1;
    digit1 = LED_N_7; 
    low_band_limit = 18068000;
    high_band_limit = 18500000;
    if (band_temp_17 != 0) {OPfreq = band_temp_17;}
    else OPfreq = 18096000;
}

void BAND15() {  
    digit2 = LED_N_1;
    digit1 = LED_N_5; 
    low_band_limit = 21000000;
    high_band_limit = 21500000;
    if (band_temp_15 != 0) {OPfreq = band_temp_15;}
    else OPfreq = 21060000;
}

void BAND12() {  
    digit2 = LED_N_1;
    digit1 = LED_N_2; 
    low_band_limit = 24890000;
    high_band_limit = 24990000;
    if (band_temp_12 != 0) {OPfreq = band_temp_12;}
    else OPfreq = 24906000;
}

void BAND10() {  
    digit2 = LED_N_1;
    digit1 = LED_N_0; 
    low_band_limit =  
    28000000;
    high_band_limit = 30000000;
    if (band_temp_10 != 0) {OPfreq = band_temp_10;}
    else OPfreq = 28060000;
}

void BAND06() {  
    digit2 = LED_N_0;
    digit1 = LED_N_6; 
    low_band_limit = 50000000;
    high_band_limit = 55000000;
    if (band_temp_06 != 0) {OPfreq = band_temp_06;}
    else OPfreq = 50060000;
}

//this compact and stand alone si5351 frequency calculation routines 
//are Copyright 2017, Jerry Gaffke, KE7ER, under the GNU General Public License 3.0 


void si5351bx_init() {                  // Call once at power-up, start PLLA
    uint8_t reg;  uint32_t msxp1;
    Wire.begin();  
    i2cWrite(149, 0);                   // SpreadSpectrum off
    i2cWrite(3, si5351bx_clken);        // Disable all CLK output drivers
    i2cWrite(183, SI5351BX_XTALPF<<6 | 0x12);  // Set 25mhz crystal load capacitance
    msxp1 = 128*SI5351BX_MSA - 512;     // and msxp2=0, msxp3=1, not fractional
    uint8_t  vals[8] = {0, 1, BB2(msxp1), BB1(msxp1), BB0(msxp1), 0, 0, 0};
    i2cWriten(26, vals, 8);             // Write to 8 PLLA msynth regs
    i2cWrite(177, 0x20);                // Reset PLLA  (0x80 resets PLLB)
    // for (reg=16; reg<=23; reg++) i2cWrite(reg, 0x80);    // Powerdown CLK's
    // i2cWrite(187, 0);                // No fannout of clkin, xtal, ms0, ms4
}

void si5351bx_setfreq(uint8_t clknum, uint32_t fout) {  // Set a CLK to fout Hz
    uint32_t  msa, msb, msc, msxp1, msxp2, msxp3p2top;
    if ((fout<500000) || (fout>109000000))  // If clock freq out of range
        si5351bx_clken |= 1<<clknum;        //  shut down the clock
    else {
        msa = si5351bx_vcoa / fout;     // Integer part of vco/fout
        msb = si5351bx_vcoa % fout;     // Fractional part of vco/fout 
        msc = fout;             // Divide by 2 till fits in reg
        while (msc & 0xfff00000) {msb=msb>>1; msc=msc>>1;}
        msxp1 =(128*msa + 128*msb/msc - 512) | (((uint32_t)si5351bx_rdiv)<<20);
        msxp2 = 128*msb - 128*msb/msc * msc;    // msxp3 == msc;        
        msxp3p2top = (((msc & 0x0F0000) <<4) | msxp2);      // 2 top nibbles
        uint8_t vals[8] = { BB1(msc), BB0(msc), BB2(msxp1), BB1(msxp1), 
            BB0(msxp1), BB2(msxp3p2top), BB1(msxp2), BB0(msxp2) };
        i2cWriten(42+(clknum*8), vals, 8);  // Write to 8 msynth regs
        i2cWrite(16+clknum, 0x0C|si5351bx_drive[clknum]);   // use local msynth
    }     
        if (clknum == 0) {si5351bx_clken = 0xfe;} //turn on the approperate clock output. 
        if (clknum == 1) {si5351bx_clken = 0xfd;}
//        si5351bx_clken &= ~(1<<clknum);     // Clear bit to enable clock   
        i2cWrite(3, si5351bx_clken);        // Enable/disable clock
}

void i2cWrite(uint8_t reg, uint8_t val){    // write reg via i2c
    Wire.beginTransmission(SI5351BX_ADDR);
    Wire.write(reg);
    Wire.write(val);
    Wire.endTransmission();
}

void i2cWriten(uint8_t reg, uint8_t *vals, uint8_t vcnt){   // write array
    Wire.beginTransmission(SI5351BX_ADDR);
    Wire.write(reg);
    while (vcnt--) Wire.write(*vals++);
    Wire.endTransmission();
}

void reset_flash(void) {
    EEPROM.write(11, 0xff);
    EEPROM.write(10, 0xff);
    EEPROM.write(9, 0xff);
    EEPROM.write(8, 0xff);
    EEPROM.write(7, 0xff);
    EEPROM.write(6, 0xff);
    EEPROM.write(5, 0xff);
    EEPROM.write(4, 0xff);

    EEPROM.write(0, 0xff);

    void (*cold_reset)(void) = 0;
    cold_reset();
}
